<?php

/*
Copyright (c) goes here...
*/

class third_party_dns_plugin {

	var $plugin_name = 'third_party_dns_plugin';
	var $class_name  = 'third_party_dns_plugin';

	//* This function is called during ispconfig installation to determine
	//  if a symlink shall be created for this plugin.
	function onInstall() {
		global $conf;

		if(@is_link('/usr/local/ispconfig/server/mods-enabled/dns_module.inc.php')) {
			return true;
		} else {
			return false;
		}

	}


	/*
		This function is called when the plugin is loaded
	*/

	function onLoad() {
		global $app;

		/*
		Register for the events
		*/

		//* SOA
		$app->plugins->registerEvent('dns_soa_insert', $this->plugin_name, 'soa_insert');
		$app->plugins->registerEvent('dns_soa_update', $this->plugin_name, 'soa_update');
		$app->plugins->registerEvent('dns_soa_delete', $this->plugin_name, 'soa_delete');

		//* SLAVE - Not necessary for CF?
		// $app->plugins->registerEvent('dns_slave_insert', $this->plugin_name, 'slave_insert');
		// $app->plugins->registerEvent('dns_slave_update', $this->plugin_name, 'slave_update');
		// $app->plugins->registerEvent('dns_slave_delete', $this->plugin_name, 'slave_delete');

		//* RR
		$app->plugins->registerEvent('dns_rr_insert', $this->plugin_name, 'rr_insert');
		$app->plugins->registerEvent('dns_rr_update', $this->plugin_name, 'rr_update');
		$app->plugins->registerEvent('dns_rr_delete', $this->plugin_name, 'rr_delete');

	}

	function soa_insert($event_name, $data) {
		global $app, $conf;

		$status = false;

		$third_party_id = $data['new']['third_party_id'];
		if (!empty($third_party_id) && !is_null($third_party_id)) {
			$third_party_connection = $app->db->queryOneRecord("SELECT * FROM third_party_connections WHERE id = ?", $third_party_id);

			if ($third_party_connection['type'] == 'cloudflare_dns') {
				$app->uses('cloudflare');
				$status = $app->cloudflare->soa_insert($event_name, $data, $third_party_connection);
			}
		}

		unset($third_party_id);
		unset($third_party_connection);
		return $status;
	}

	function soa_update($event_name, $data) {
		global $app, $conf;

		$status = false;

		$third_party_id = $data['new']['third_party_id'];
		if (!empty($third_party_id) && !is_null($third_party_id)) {
			$third_party_connection = $app->db->queryOneRecord("SELECT * FROM third_party_connections WHERE id = ?", $third_party_id);

			if ($third_party_connection['type'] == 'cloudflare_dns') {
				$app->uses('cloudflare');
				$status = $app->cloudflare->soa_update($event_name, $data, $third_party_connection);
			}
		}

		unset($third_party_id);
		unset($third_party_connection);
		return $status;
	}

	function soa_delete($event_name, $data) {
		global $app, $conf;

		$status = false;

		$third_party_id = $data['new']['third_party_id'];
		if (!empty($third_party_id) && !is_null($third_party_id)) {
			$third_party_connection = $app->db->queryOneRecord("SELECT * FROM third_party_connections WHERE id = ?", $third_party_id);

			if ($third_party_connection['type'] == 'cloudflare_dns') {
				$app->uses('cloudflare');
				$status = $app->cloudflare->soa_delete($event_name, $data, $third_party_connection);
			}
		}

		unset($third_party_id);
		unset($third_party_connection);
		return $status;
	}

	function rr_insert($event_name, $data) {
		global $app, $conf;

		$status = false;

		$third_party_id = $app->db->queryOneRecord("SELECT third_party_id FROM dns_soa WHERE id = ?", $data['new']['zone']);
		if (!empty($third_party_id) && !is_null($third_party_id)) {
			$third_party_connection = $app->db->queryOneRecord("SELECT * FROM third_party_connections WHERE id = ?", $third_party_id);

			if ($third_party_connection['type'] == 'cloudflare_dns') {
				$app->uses('cloudflare');
				$status = $app->cloudflare->rr_insert($event_name, $data, $third_party_connection);
			}
		}

		unset($third_party_id);
		unset($third_party_connection);
		return $status;
	}

	function rr_update($event_name, $data) {
		global $app, $conf;

		$status = false;

		$third_party_id = $app->db->queryOneRecord("SELECT third_party_id FROM dns_soa WHERE id = ?", $data['new']['zone']);
		if (!empty($third_party_id) && !is_null($third_party_id)) {
			$third_party_connection = $app->db->queryOneRecord("SELECT * FROM third_party_connections WHERE id = ?", $third_party_id);

			if ($third_party_connection['type'] == 'cloudflare_dns') {
				$app->uses('cloudflare');
				$status = $app->cloudflare->rr_update($event_name, $data, $third_party_connection);
			}
		}

		unset($third_party_id);
		unset($third_party_connection);
		return $status;
	}

	function rr_delete($event_name, $data) {
		global $app, $conf;

		$status = false;

		$third_party_id = $app->db->queryOneRecord("SELECT third_party_id FROM dns_soa WHERE id = ?", $data['new']['zone']);
		if (!empty($third_party_id) && !is_null($third_party_id)) {
			$third_party_connection = $app->db->queryOneRecord("SELECT * FROM third_party_connections WHERE id = ?", $third_party_id);

			if ($third_party_connection['type'] == 'cloudflare_dns') {
				$app->uses('cloudflare');
				$status = $app->cloudflare->rr_delete($event_name, $data, $third_party_connection);
			}
		}

		unset($third_party_id);
		unset($third_party_connection);
		return $status;
	}

//	Not registered for events
//
//	function slave_insert($event_name, $data) {
//		global $app, $conf;
//
//		$app->log("third_party_dns_plugin: ".__METHOD__, LOGLEVEL_DEBUG);
//	}
//
//	function slave_update($event_name, $data) {
//		global $app, $conf;
//
//		$app->log("third_party_dns_plugin: ".__METHOD__, LOGLEVEL_DEBUG);
//	}
//
//	function slave_delete($event_name, $data) {
//		global $app, $conf;
//
//		$app->log("third_party_dns_plugin: ".__METHOD__, LOGLEVEL_DEBUG);
//	}

	//##################################################################

} // end class

?>
